@@ -8,7 +8,6 @@
 '''A simple Twitter client made with pyqt4'''
 
 import httplib2
-import re
 import os
 
 # import sip
@@ -22,6 +21,10 @@ from PySide.QtGui import QMainWindow, QSizePolicy, QSpinBox, QVBoxLayout, \
 from PySide.QtCore import Qt, QUrl, QAbstractListModel, QSettings, QModelIndex, \
     Signal
 
+from theme import DEFAULTTHEME, WHITETHEME, \
+                     COOLWHITETHEME, COOLGRAYTHEME, \
+                     MINITHEME
+
 SUPPORTED_ACCOUNTS = [{
     'name': 'Twitter',
     'consumer_key': 'uhgjkoA2lggG4Rh0ggUeQ',
@@ -48,7 +51,7 @@ try:
 except:
     from cgi import parse_qs
 
-from PySide.QtWebKit import *
+from PySide.QtWebKit import QWebView
 
 
 class OAuthView(QWebView):
@@ -81,6 +84,7 @@ class OAuthView(QWebView):
         return self
 
     def _loadFinished(self):
+        import re
 
         regex = re.compile('.*<div.*oauth_pin.*>(.*)<')
         res = regex.findall(self.page().mainFrame().toHtml())
@@ -186,11 +190,6 @@ class KhweeteurPref(QMainWindow):
 
     save = Signal()
 
-    DEFAULTTHEME = u'Default'
-    WHITETHEME = u'White'
-    COOLWHITETHEME = u'CoolWhite'
-    COOLGRAYTHEME = u'CoolGray'
-    MINITHEME = u'MiniDefault'
     THEMES = [DEFAULTTHEME, WHITETHEME, COOLWHITETHEME, COOLGRAYTHEME,
               MINITHEME]
 
@@ -286,11 +285,20 @@ class KhweeteurPref(QMainWindow):
             self.useGPSOnDemand_value.setCheckState(Qt.CheckState(2))
 
         if self.settings.contains('showInfos'):
-            self.showInfos_value.setCheckState(Qt.CheckState(int(self.settings.value('showInfos'
-                    ))))
+            self.showInfos_value.setCheckState(Qt.CheckState(int(self.settings.value('showInfos'))))
         else:
             self.showInfos_value.setCheckState(Qt.CheckState(0))
 
+        if self.settings.contains('showDMNotifications'):
+            self.showDMNotifications_value.setCheckState(Qt.CheckState(int(self.settings.value('showDMNotifications'))))
+        else:
+            self.showDMNotifications_value.setCheckState(Qt.CheckState(2))
+
+        if self.settings.contains('showMentionNotifications'):
+            self.showMentionNotifications_value.setCheckState(Qt.CheckState(int(self.settings.value('showMentionNotifications'))))
+        else:
+            self.showMentionNotifications_value.setCheckState(Qt.CheckState(2))
+
     def savePrefs(self):
         ''' Save the prefs from the GUI to QSettings'''
 
@@ -316,6 +324,8 @@ class KhweeteurPref(QMainWindow):
         self.settings.setValue('useGPS', self.useGPS_value.checkState())
         self.settings.setValue('useGPSOnDemand', self.useGPSOnDemand_value.checkState())
         self.settings.setValue('showInfos', self.showInfos_value.checkState())
+        self.settings.setValue('showDMNotifications', self.showDMNotifications_value.checkState())
+        self.settings.setValue('showMentionNotifications', self.showMentionNotifications_value.checkState())
         self.settings.setValue('tweetHistory', self.history_value.value())
         self.settings.sync()
         self.save.emit()
@@ -502,6 +512,12 @@ class KhweeteurPref(QMainWindow):
 
         self.showInfos_value = QCheckBox(self.tr('Show errors notifications'))
         self._umain_layout.addWidget(self.showInfos_value, 15, 1)
+        
+        self.showDMNotification_value = QCheckBox(self.tr('Use DMs notifications'))
+        self._umain_layout.addWidget(self.showDMNotification_value, 16, 1)
+        
+        self.showMentionNotification_value = QCheckBox(self.tr('Use Mentions notifications'))
+        self._umain_layout.addWidget(self.showMentionNotification_value, 17, 1)
 
         self._main_layout.addLayout(self._umain_layout)
 
