@@ -9,7 +9,7 @@
 
 from __future__ import with_statement
 
-__version__ = '0.5.14'
+__version__ = '0.5.21'
 
 # import sip
 # sip.setapi('QString', 2)
@@ -18,72 +18,31 @@ __version__ = '0.5.14'
 from PySide.QtGui import QMainWindow, QHBoxLayout, QSizePolicy, QToolButton, \
     QVBoxLayout, QFileDialog, QDesktopServices, QScrollArea, QPushButton, \
     QToolBar, QLabel, QWidget, QInputDialog, QMenu, QAction, QApplication, \
-    QIcon, QMessageBox, QPlainTextEdit
+    QIcon, QMessageBox, QPlainTextEdit, QTextCursor
 from PySide.QtCore import Qt, QUrl, QSettings, Slot, Signal, QTimer
-from PySide.QtMaemo5 import *
+
+try:
+    from PySide.QtMaemo5 import *
+except:
+    pass
+
+try:
+    import osso
+except:
+    pass
+
 from qbadgebutton import QToolBadgeButton
 
-import dbus
-import dbus.service
 import os
 import sys
-import pickle
-import time
 from list_view import KhweetsView
 from list_model import KhweetsModel, ISMEROLE, IDROLE, ORIGINROLE, SCREENNAMEROLE, PROTECTEDROLE, USERIDROLE
-from settings import KhweeteurPref
-from dbusobj import KhweeteurDBus
-import re
+
 
 try:
     from QtMobility.Location import QGeoPositionInfoSource
 except:
     print 'Pyside QtMobility not installed or broken'
-    
-class KhweeteurDBusHandler(dbus.service.Object):
-
-    def __init__(self, parent):
-        dbus.service.Object.__init__(self, dbus.SessionBus(),
-                                     '/net/khertan/Khweeteur')
-        self.parent = parent
-
-        # Post Folder
-
-        self.post_path = os.path.join(os.path.expanduser('~'), '.khweeteur',
-                                      'topost')
-
-    @dbus.service.signal(dbus_interface='net.khertan.Khweeteur')
-    def require_update(self, optional=None):
-        self.parent.setAttribute(Qt.WA_Maemo5ShowProgressIndicator, True)
-
-    def post_tweet(
-        self,
-        shorten_url=1,
-        serialize=1,
-        text='',
-        lattitude='0',
-        longitude='0',
-        base_url='',
-        action='',
-        tweet_id='0',
-        ):
-
-        if not os.path.exists(self.post_path):
-            os.makedirs(self.post_path)
-        with open(os.path.join(self.post_path, str(time.time())), 'wb') as \
-            fhandle:
-            post = {
-                'shorten_url': shorten_url,
-                'serialize': serialize,
-                'text': text,
-                'lattitude': lattitude,
-                'longitude': longitude,
-                'base_url': base_url,
-                'action': action,
-                'tweet_id': tweet_id,
-                }
-            pickle.dump(post, fhandle, pickle.HIGHEST_PROTOCOL)
-
 
 class KhweeteurAbout(QMainWindow):
 
@@ -96,12 +55,11 @@ class KhweeteurAbout(QMainWindow):
         self.settings = QSettings()
 
         try:  # Preferences not set yet
-            if int(self.settings.value('useAutoRotation')) == 2:
-                self.setAttribute(Qt.WA_Maemo5AutoOrientation, True)
-        except:
             self.setAttribute(Qt.WA_Maemo5AutoOrientation, True)
+            self.setAttribute(Qt.WA_Maemo5StackedWindow, True)
+        except:
+            pass
 
-        self.setAttribute(Qt.WA_Maemo5StackedWindow, True)
         self.setWindowTitle(self.tr('Khweeteur About'))
 
         try:
@@ -210,6 +168,11 @@ class Khweeteur(QApplication):
         self.setOrganizationName('Khertan Software')
         self.setOrganizationDomain('khertan.net')
         self.setApplicationName('Khweeteur')
+
+        try:
+            self.osso_c = osso.Context('net.khertan.khweeteur', "0.0.1", False)
+        except:
+            pass
         self.run()
 
     def check_crash_report(self):
@@ -267,8 +230,6 @@ class Khweeteur(QApplication):
     def run(self):
         self.check_crash_report()
         self.win = KhweeteurWin()
-        self.win.show()
-
 
 class KhweeteurWin(QMainWindow):
 
@@ -277,15 +238,19 @@ class KhweeteurWin(QMainWindow):
     def __init__(self, parent=None):
         QMainWindow.__init__(self, parent)
 
-        self.setAttribute(Qt.WA_Maemo5AutoOrientation, True)
-        self.setAttribute(Qt.WA_Maemo5StackedWindow, True)
-        self.setWindowTitle('Khweeteur')
+        try:
+            self.setAttribute(Qt.WA_Maemo5AutoOrientation, True)
+            self.setAttribute(Qt.WA_Maemo5StackedWindow, True)
+        except:
+            pass
 
-        self.listen_dbus()
+        self.setWindowTitle('Khweeteur')
 
         settings = QSettings()
 
         self.view = KhweetsView()
+        self.setCentralWidget(self.view)
+
         self.model = KhweetsModel()
         try:
             self.model.setLimit(int(settings.value('tweetHistory')))
@@ -294,8 +259,6 @@ class KhweeteurWin(QMainWindow):
         self.view.setModel(self.model)
         self.view.clicked.connect(self.switch_tb_action)
 
-        self.dbus_handler.require_update()
-
         self.toolbar = QToolBar('Toolbar')
         self.addToolBar(Qt.BottomToolBarArea, self.toolbar)
 
@@ -312,51 +275,10 @@ class KhweeteurWin(QMainWindow):
         self.toolbar.addAction(self.tb_new)
         self.list_tb_action.append(self.tb_new)
 
-        # Back button (Edit + Action)
-
-        self.tb_back = QAction(QIcon.fromTheme('general_back'), 'Back', self)
-        self.tb_back.triggered.connect(self.switch_tb_default)
-        self.toolbar.addAction(self.tb_back)
-        self.edit_tb_action.append(self.tb_back)
-        self.action_tb_action.append(self.tb_back)
-
-        self.setupMenu()
-
-        # Twitpic button
-
-        self.tb_twitpic = QAction(QIcon.fromTheme('tasklaunch_images'),
-                                  'Twitpic', self)
-        self.tb_twitpic.triggered.connect(self.do_tb_twitpic)
-        self.toolbar.addAction(self.tb_twitpic)
-        self.edit_tb_action.append(self.tb_twitpic)
-
-        # Text field (edit)
-
-        self.tb_text = QPlainTextEdit()
-        self.tb_text_reply_id = 0
-        self.tb_text_reply_base_url = ''
-        self.tb_text.setFixedHeight(66)
-        self.edit_tb_action.append(self.toolbar.addWidget(self.tb_text))
-
-        # Char count (Edit)
-
-        self.tb_charCounter = QLabel('140')
-        self.edit_tb_action.append(self.toolbar.addWidget(self.tb_charCounter))
-        self.tb_text.textChanged.connect(self.countCharsAndResize)
-
-        # Send tweet (Edit)
-
-        self.tb_send = QAction(QIcon.fromTheme('khweeteur'), 'Tweet', self)
-        self.tb_send.triggered.connect(self.do_tb_send)
-        self.tb_send.setVisible(False)
-        self.toolbar.addAction(self.tb_send)
-        self.edit_tb_action.append(self.tb_send)
-
         # Refresh (Default)
 
         self.tb_update = QAction(QIcon.fromTheme('general_refresh'), 'Update',
                                  self)
-        self.tb_update.triggered.connect(self.dbus_handler.require_update)
         self.toolbar.addAction(self.tb_update)
         self.list_tb_action.append(self.tb_update)
 
@@ -388,7 +310,6 @@ class KhweeteurWin(QMainWindow):
         # Search Button
 
         self.tb_search_menu = QMenu()
-        self.loadSearchMenu()
 
         # Search (Default)
 
@@ -404,7 +325,6 @@ class KhweeteurWin(QMainWindow):
         # Lists Button
 
         self.tb_list_menu = QMenu()
-        self.loadListMenu()
 
         # Lists (Default)
 
@@ -417,6 +337,14 @@ class KhweeteurWin(QMainWindow):
         self.tb_list_button.clicked.connect(self.show_list)
         self.list_tb_action.append(self.toolbar.addWidget(self.tb_list_button))
 
+        # Near (Default)
+        self.near_button = QToolBadgeButton(self)
+        self.near_button.setText('Nears')
+        self.near_button.setIcon(QIcon.fromTheme('general_web'))
+        self.near_button.setCheckable(True)
+        self.near_button.clicked.connect(self.show_nears)
+        self.list_tb_action.append(self.toolbar.addWidget(self.near_button))
+
         # Fullscreen
 
         self.tb_fullscreen = QAction(QIcon.fromTheme('general_fullsize'),
@@ -425,33 +353,119 @@ class KhweeteurWin(QMainWindow):
         self.toolbar.addAction(self.tb_fullscreen)
         self.list_tb_action.append(self.tb_fullscreen)
 
+        self.setWindowTitle('Khweeteur : Home')
+
+        self.show()
+
+        self.model.load('HomeTimeline',limit=5)
+        QTimer.singleShot(100,self.post_init_2)
+        QTimer.singleShot(500,self.post_init_3)
+        QApplication.processEvents()
+
+    @Slot()
+    def post_init_3(self):
+        self.model.load('HomeTimeline')
+        #Check if there is at least one account
+        nb_accounts = QSettings().beginReadArray('accounts')
+        if not nb_accounts:
+            if not (( QMessageBox.question(None,
+                "Kheeteur",
+                'None microblogging account found, would you want to add one now in preferences ?',
+                QMessageBox.Yes| QMessageBox.Close)) ==  QMessageBox.Yes):
+                    self.showPrefs()
+
+    @Slot()
+    def post_init_2(self):
+        self.listen_dbus()
+        self.dbus_handler.require_update()
+        self.tb_update.triggered.connect(self.dbus_handler.require_update)
+        self.geolocDoStart()
+
+        self.loadSearchMenu()
+        self.loadListMenu()
+
+        #Toolbar to set after startup
+        # Back button (Edit + Action)
+
+        self.tb_back = QAction(QIcon.fromTheme('general_back'), 'Back', self)
+        self.tb_back.triggered.connect(self.switch_tb_default)
+        self.toolbar.addAction(self.tb_back)
+        self.tb_back.setVisible(False)
+        self.edit_tb_action.append(self.tb_back)
+        self.action_tb_action.append(self.tb_back)
+
+        self.setupMenu()
+
+        # Twitpic button
+
+        self.tb_twitpic = QAction(QIcon.fromTheme('tasklaunch_images'),
+                                  'Twitpic', self)
+        self.tb_twitpic.triggered.connect(self.do_tb_twitpic)
+        self.tb_twitpic.setVisible(False)
+        self.toolbar.addAction(self.tb_twitpic)
+        self.edit_tb_action.append(self.tb_twitpic)
+
+        # Text field (edit)
+
+        self.tb_text = QPlainTextEdit()
+        self.tb_text_reply_id = 0
+        self.tb_text_reply_base_url = ''
+        self.tb_text.setFixedHeight(66)
+        action = self.toolbar.addWidget(self.tb_text)
+        action.setVisible(False)
+        self.edit_tb_action.append(action)
+
+        # Char count (Edit)
+
+        self.tb_charCounter = QLabel('140')
+        action = self.toolbar.addWidget(self.tb_charCounter)
+        action.setVisible(False)
+        self.edit_tb_action.append(action)
+        self.tb_text.textChanged.connect(self.countCharsAndResize)
+
+        # Send tweet (Edit)
+
+        self.tb_send = QAction(QIcon.fromTheme('khweeteur'), 'Tweet', self)
+        self.tb_send.triggered.connect(self.do_tb_send)
+        self.tb_send.setVisible(False)
+        self.toolbar.addAction(self.tb_send)
+        self.edit_tb_action.append(self.tb_send)
+
         # Reply button (Action)
 
         self.tb_reply = QAction('Reply', self)
         self.tb_reply.setShortcut('Ctrl+M')
+        self.tb_reply.setVisible(False)
         self.toolbar.addAction(self.tb_reply)
         self.tb_reply.triggered.connect(self.do_tb_reply)
         self.action_tb_action.append(self.tb_reply)
 
         # Retweet (Action)
-
         self.tb_retweet = QAction('Retweet', self)
         self.tb_retweet.setShortcut('Ctrl+P')
+        self.tb_retweet.setVisible(False)
         self.toolbar.addAction(self.tb_retweet)
         self.tb_retweet.triggered.connect(self.do_tb_retweet)
         self.action_tb_action.append(self.tb_retweet)
 
-        # Follow (Action)
+        # RT (Action)
+        self.tb_rt = QAction('RT', self)
+        self.tb_rt.setVisible(False)
+        self.toolbar.addAction(self.tb_rt)
+        self.tb_rt.triggered.connect(self.do_tb_rt)
+        self.action_tb_action.append(self.tb_rt)
 
+        # Follow (Action)
         self.tb_follow = QAction('Follow', self)
         self.tb_follow.triggered.connect(self.do_tb_follow)
+        self.tb_follow.setVisible(False)
         self.toolbar.addAction(self.tb_follow)
         self.action_tb_action.append(self.tb_follow)
 
         # UnFollow (Action)
-
         self.tb_unfollow = QAction('Unfollow', self)
         self.tb_unfollow.triggered.connect(self.do_tb_unfollow)
+        self.tb_unfollow.setVisible(False)
         self.toolbar.addAction(self.tb_unfollow)
         self.action_tb_action.append(self.tb_unfollow)
 
@@ -459,13 +473,15 @@ class KhweeteurWin(QMainWindow):
 
         self.tb_favorite = QAction('Favorite', self)
         self.tb_favorite.triggered.connect(self.do_tb_favorite)
+        self.tb_favorite.setVisible(False)
         self.toolbar.addAction(self.tb_favorite)
         self.action_tb_action.append(self.tb_favorite)
 
         # Open URLs (Action)
 
-        self.tb_urls = QAction('Open URLs', self)
+        self.tb_urls = QAction('URLs', self)
         self.tb_urls.setShortcut('Ctrl+O')
+        self.tb_urls.setVisible(False)
         self.toolbar.addAction(self.tb_urls)
         self.tb_urls.triggered.connect(self.do_tb_openurl)
         self.action_tb_action.append(self.tb_urls)
@@ -473,6 +489,7 @@ class KhweeteurWin(QMainWindow):
         # Delete (Action)
 
         self.tb_delete = QAction('Delete', self)
+        self.tb_delete.setVisible(False)
         self.toolbar.addAction(self.tb_delete)
         self.tb_delete.triggered.connect(self.do_tb_delete)
         self.action_tb_action.append(self.tb_delete)
@@ -504,17 +521,6 @@ class KhweeteurWin(QMainWindow):
         self.tb_copy.triggered.connect(self.do_tb_copy)
         self.addAction(self.tb_copy)
 
-        self.switch_tb_default()
-
-        self.setWindowTitle('Khweeteur : Home')
-        self.setCentralWidget(self.view)
-        QTimer.singleShot(100,self.post_init)
-
-    @Slot()
-    def post_init(self):
-        self.model.load('HomeTimeline')
-        self.geolocDoStart()
-
     @Slot()
     def do_tb_fullscreen(self):
         if self.isFullScreen():
@@ -540,6 +546,10 @@ class KhweeteurWin(QMainWindow):
         self.model.refreshTimestamp()
 
     def listen_dbus(self):
+        import dbus
+        import dbus.service
+        from dbusobj import KhweeteurDBus
+        from dbushandler import KhweeteurDBusHandler
         from dbus.mainloop.qt import DBusQtMainLoop
         self.dbus_loop = DBusQtMainLoop()
         dbus.set_default_main_loop(self.dbus_loop)
@@ -561,7 +571,10 @@ class KhweeteurWin(QMainWindow):
         dbusobj.attach_win(self)
 
     def stop_spinning(self):
-        self.setAttribute(Qt.WA_Maemo5ShowProgressIndicator, False)
+        try:
+            self.setAttribute(Qt.WA_Maemo5ShowProgressIndicator, False)
+        except AttributeError:
+            pass
 
     def new_tweets(self, count, msg):
         if msg == 'HomeTimeline':
@@ -574,6 +587,9 @@ class KhweeteurWin(QMainWindow):
         elif msg == 'DMs':
             self.msg_button.setCounter(self.msg_button.getCounter() + count)
             self.msg_button.update()
+        elif msg == 'Nears':
+            self.near_button.setCounter(self.near_button.getCounter() + count)
+            self.near_button.update()
         elif msg.startswith('Search:'):
             self.tb_search_button.setCounter(self.tb_search_button.getCounter()
                     + count)
@@ -595,6 +611,7 @@ class KhweeteurWin(QMainWindow):
         self.tb_search_button.setChecked(True)
         self.mention_button.setChecked(False)
         self.tb_list_button.setChecked(False)
+        self.near_button.setChecked(False)
         self.view.scrollToTop()
         self.model.load('Search:' + terms)
         self.delete_search_action.setVisible(True)
@@ -612,6 +629,7 @@ class KhweeteurWin(QMainWindow):
         self.tb_search_button.setChecked(False)
         self.tb_list_button.setChecked(True)
         self.mention_button.setChecked(False)
+        self.near_button.setChecked(False)
         self.view.scrollToTop()
         self.model.load('List:' + user + ':' + tid)
         self.delete_search_action.setVisible(True)
@@ -623,6 +641,7 @@ class KhweeteurWin(QMainWindow):
         self.home_button.setChecked(True)
         self.msg_button.setChecked(False)
         self.tb_search_button.setChecked(False)
+        self.near_button.setChecked(False)
         self.tb_list_button.setChecked(False)
         self.mention_button.setChecked(False)
         self.view.scrollToTop()
@@ -702,8 +721,8 @@ class KhweeteurWin(QMainWindow):
                 0,
                 1,
                 text,
-                ('' if self.geoloc_source == None else self.geoloc_source[0]),
-                ('' if self.geoloc_source == None else self.geoloc_source[1]),
+                ('' if self.geoloc_source == None else self.geoloc_coordinates[0]),
+                ('' if self.geoloc_source == None else self.geoloc_coordinates[1]),
                 filename,
                 'twitpic',
                 '',
@@ -713,6 +732,7 @@ class KhweeteurWin(QMainWindow):
 
     @Slot()
     def do_tb_openurl(self):
+        import re
         for index in self.view.selectedIndexes():
             status = self.model.data(index)
             try:
@@ -753,12 +773,31 @@ class KhweeteurWin(QMainWindow):
             tweet_source = self.model.data(index, role=ORIGINROLE)
             tweet_screenname = self.model.data(index, role=SCREENNAMEROLE)
         if tweet_id:
-            self.tb_text.setPlainText('@' + tweet_screenname
-                                      + self.tb_text.toPlainText())
+            self.tb_text.setPlainText('@%s ' % tweet_screenname)
+            cur = self.tb_text.textCursor()
+            cur.movePosition(QTextCursor.End, QTextCursor.MoveAnchor)
+            self.tb_text.setTextCursor(cur)
             self.tb_text_reply_id = tweet_id
             self.tb_text_reply_base_url = tweet_source
             self.switch_tb_edit()
 
+    @Slot()
+    def do_tb_rt(self):
+        tweet_id = None
+        for index in self.view.selectedIndexes():
+            tweet_id = self.model.data(index, role=IDROLE)
+            tweet_screenname = self.model.data(index, role=SCREENNAMEROLE)
+            tweet_text = self.model.data(index, role=Qt.DisplayRole)
+        if tweet_id:
+            self.tb_text.setPlainText('RT @%s %s' % (tweet_screenname, tweet_text))
+            cur = self.tb_text.textCursor()
+            cur.movePosition(QTextCursor.End, QTextCursor.MoveAnchor)
+            self.tb_text.setTextCursor(cur)
+            self.switch_tb_edit()
+            self.tb_text.show()
+            self.tb_text.updateGeometry()
+            self.tb_text.textChanged.emit()
+            
     @Slot()
     def do_tb_retweet(self):
 
@@ -943,6 +982,7 @@ class KhweeteurWin(QMainWindow):
         self.tb_list_button.setChecked(False)
         self.tb_search_button.setChecked(False)
         self.home_button.setChecked(False)
+        self.near_button.setChecked(False)
         self.view.scrollToTop()
         self.model.load('Mentions')
         self.setWindowTitle('Khweeteur : Mentions')
@@ -956,11 +996,37 @@ class KhweeteurWin(QMainWindow):
         self.home_button.setChecked(False)
         self.tb_search_button.setChecked(False)
         self.mention_button.setChecked(False)
+        self.near_button.setChecked(False)
         self.view.scrollToTop()
         self.model.load('DMs')
         self.setWindowTitle('Khweeteur : DMs')
         self.delete_search_action.setVisible(False)
 
+    @Slot()
+    def show_nears(self):
+        settings = QSettings()
+        if settings.value('useGPS') != '2':
+            if (( QMessageBox.question(None,
+                "Kheeteur",
+                'This feature require the activation of the GPS, would you want to activate it now ?',
+                QMessageBox.Yes| QMessageBox.Close)) ==  QMessageBox.Yes):
+                    settings.setValue('useGPS','2')
+                    settings.sync()
+                    self.geolocDoStart()
+                    self.dbus_handler.require_update()
+
+        self.near_button.setCounter(0)
+        self.near_button.setChecked(True)
+        self.msg_button.setChecked(False)
+        self.tb_list_button.setChecked(False)
+        self.home_button.setChecked(False)
+        self.tb_search_button.setChecked(False)
+        self.mention_button.setChecked(False)
+        self.view.scrollToTop()
+        self.model.load('Nears')
+        self.setWindowTitle('Khweeteur : Near Tweets')
+        self.delete_search_action.setVisible(False)
+
     @Slot()
     def countCharsAndResize(self):
         local_self = self.tb_text
@@ -968,41 +1034,14 @@ class KhweeteurWin(QMainWindow):
                                     - len(local_self.toPlainText())))
         doc = local_self.document()
         fm = local_self.fontMetrics()
-#        line_height = fm.boundingRect(local_self.toPlainText()).height()
 
-#        height = (line_height ) * doc.documentLayout().documentSize().height() + fm.lineSpacing()
-#        s = doc.documentLayout().documentSize()
-
-#        print 'Doc size', doc.size().height()
-
-#        doc.setHeight(s.height())
-#        s.setHeight((s.height() + 2) + (local_self.fontMetrics().lineSpacing()*2 + 2))
-#        s.setHeight(s.height())
-#        print 'Doc size', s.height()
-#        fr = local_self.frameRect()
-#        print 'frame size', fr.size().height()
-#        cr = local_self.contentsRect()
-#        print 'content size', cr.size().height()
-#        print 'page size', doc.pageSize().height()
-#        print 'page count', doc.pageCount()
-#        local_self.setFixedHeight(min(370, s.height() + fr.height()
-#                                  - cr.height() - 1))
-#        text_height = fm.boundingRect(0,0,local_self.size().width(),370, \
-#                    int(Qt.AlignTop) | int(Qt.AlignLeft) | int(Qt.TextWordWrap), \
-#                    local_self.toPlainText()).height()
-#        print 'text height',text_height
-#        if height > 5 :
-#        local_self.setFixedHeight(min(370, text_height + (fr.height() - cr.height()) + 6))
-#        local_self.updateGeometry()
-#Resize
-#        doc = self.document()            
         s = doc.size().toSize()
         s.setHeight((s.height() + 1) * (fm.lineSpacing()+1))
         fr = local_self.frameRect()
         cr = local_self.contentsRect()
         local_self.setFixedHeight(min(s.height() +  (fr.height() - cr.height() - 1) - 15,370))
         local_self.updateGeometry()
-        
+
     def loadSearchMenu(self):
         settings = QSettings()
         self.tb_search_menu.clear()
@@ -1085,6 +1124,7 @@ class KhweeteurWin(QMainWindow):
 
     @Slot()
     def showPrefs(self):
+        from settings import KhweeteurPref
         khtsettings = KhweeteurPref(parent=self)
         khtsettings.save.connect(self.refreshPrefs)
         khtsettings.show()
@@ -1104,7 +1144,7 @@ class KhweeteurWin(QMainWindow):
         settings = QSettings()
         self.geoloc_source = None
         if settings.contains('useGPS'):
-            if settings.value('useGPS') == '2':
+            if (settings.value('useGPS') == '2') and (settings.value('useGPSOnDemand')!='2'):
                 self.geolocStart()
             else:
                 self.geolocStop()
